
/* /\****************************** */
/*  * Form Wrapper */
/*  ******************************\/ */

/* div.form-wrapper { */
/*     overflow: auto; */
/* } */


/******************************
 * context menu
 ******************************/

/* #context-menu { */
/*     /\* background-color: #ddcccc; *\/ */
/*     /\* background-color: green; *\/ */
/*     float: right; */
/*     /\* list-style-type: none; *\/ */
/*     /\* margin: 0px; *\/ */
/*     text-align: right; */
/* } */

/* div.form-wrapper ul.context-menu li { */
/*     line-height: 2em; */
/* } */


/* /\****************************** */
/*  * "object helper" panel */
/*  ******************************\/ */

/* .object-helper { */
/*     border: 1px solid black; */
/*     float: right; */
/*     margin-top: 1em; */
/*     padding: 1em; */
/*     width: 20em; */
/* } */

/* .object-helper-content { */
/*     margin-top: 1em; */
/* } */


/******************************
 * forms
 ******************************/

/* div.form, */
/* div.fieldset-form, */
/* div.fieldset { */
/*     clear: left; */
/*     float: left; */
/*     margin-top: 10px; */
/* } */

/* TODO: replace this with bulma equivalent */
.form {
    padding-left: 5em;
}


/******************************
 * fieldsets
 ******************************/

/* TODO: replace this with bulma equivalent */
.field-wrapper {
    clear: both;
    min-height: 30px;
    overflow: auto;
    margin: 15px;
}

/* .field-wrapper.with-error { */
/*     background-color: #ddcccc; */
/*     border: 2px solid #dd6666; */
/*     padding-bottom: 1em; */
/* } */

/* TODO: replace this with bulma equivalent */
.field-wrapper .field-row {
    display: table-row;
}

/* TODO: replace this with bulma equivalent */
.field-wrapper label {
    display: table-cell;
    vertical-align: top;
    width: 18em;
    font-weight: bold;
    padding-top: 2px;
    white-space: nowrap;
}

/* .field-wrapper.with-error label { */
/*     padding-left: 1em; */
/* } */

/* .field-wrapper .field-error { */
/*     padding: 1em 0 0.5em 1em; */
/* } */

/* .field-wrapper .field-error .error-msg { */
/*     color: #dd6666; */
/*     font-weight: bold; */
/* } */

/* TODO: replace this with bulma equivalent */
.field-wrapper .field {
    display: table-cell;
    line-height: 25px;
}

/* .field-wrapper .field input[type=text], */
/* .field-wrapper .field input[type=password], */
/* .field-wrapper .field select, */
/* .field-wrapper .field textarea { */
/*     width: 320px; */
/* } */

/* label input[type="checkbox"], */
/* label input[type="radio"] { */
/*     margin-right: 0.5em; */
/* } */

/* .field ul { */
/*     margin: 0px; */
/*     padding-left: 15px; */
/* } */


/* /\****************************** */
/*  * Buttons */
/*  ******************************\/ */

/* div.buttons { */
/*     clear: both; */
/*     margin: 10px 0px; */
/* } */
