
/******************************
 * Permission Lists
 ******************************/

.permissions-group {
    margin-bottom: 10px;
}

.permissions-group .group-label {
    font-weight: bold;
    margin-bottom: 10px;
}

.field-wrapper.permissions .field label {
    display: inline;
    font-weight: normal;
}

.field-wrapper.permissions .field label input {
    margin-left: 15px;
    margin-right: 10px;
}

.permissions-group .perm {
    font-weight: normal;
    margin-left: 15px;
}

.permissions-group p.perm span {
    font-family: monospace;
    margin-right: 10px;
}
