
/******************************
 * Main Layout
 ******************************/

html, body, #body-wrapper {
    height: 100%;
}

body > #body-wrapper {
    height: auto;
    min-height: 100%;
}

#body-wrapper {
    margin: 0 1em;
    width: auto;
}

#header {
    height: 50px;
    line-height: 50px;
}

#body {
    padding-top: 10px;
    padding-bottom: 5em;
}

#footer {
    clear: both;
    margin-top: -4em;
    text-align: center;
}


/******************************
 * Header
 ******************************/

#header h1 {
    float: left;
    font-size: 25px;
    margin: 0px;
}

#header div.login {
    float: right;
}

/* new stuff from 'better' theme begins here */

header .global {
    background-color: #eaeaea;
    height: 60px;
}

header .global a.home,
header .global a.global,
header .global span.global {
    display: block;
    float: left;
    font-size: 2em;
    font-weight: bold;
    line-height: 60px;
    margin-left: 10px;
}

header .global a.home img {
    display: block;
    float: left;
    padding: 5px 5px 5px 30px;
}

header .global .grid-nav {
    display: inline-block;
    font-size: 16px;
    font-weight: bold;
    line-height: 60px;
    margin-left: 5em;
}

header .global .grid-nav .ui-button,
header .global .grid-nav span.viewing {
    margin-left: 1em;
}

header .global .feedback {
    float: right;
    line-height: 60px;
    margin-right: 1em;
}

header .global .after-feedback {
    float: right;
    line-height: 60px;
    margin-right: 1em;
}

header .page {
    border-bottom: 1px solid lightgrey;
    padding: 0.5em;
}

header .page h1 {
    margin: 0;
    padding: 0 0 0 0.5em;
}

/******************************
 * Logo
 ******************************/

#logo {
    display: block;
    margin: 40px auto;
}


/****************************************
 * content
 ****************************************/

body > #body-wrapper {
    margin: 0px;
    position: relative;
}

.content-wrapper {
    height: 100%;
    padding-bottom: 30px;
}

#scrollpane {
    height: 100%;
}

#scrollpane .inner-content {
    padding: 0 0.5em 0.5em 0.5em;
}


/******************************
 * context menu
 ******************************/

#context-menu {
    list-style-type: none;
    margin: 0.5em;
    text-align: right;
    white-space: nowrap;
}

/******************************
 * "object helper" panel
 ******************************/

.object-helper {
    border: 1px solid black;
    margin: 1em;
    padding: 1em;
    min-width: 20em;
}

.object-helper-content {
    margin-top: 1em;
}

/******************************
 * Panels
 ******************************/

.panel,
.panel-grid {
    border-left: 1px solid Black;
    margin-bottom: 1em;
}

.panel {
    border-bottom: 1px solid Black;
    border-right: 1px solid Black;
    padding: 0px;
}

.panel h2,
.panel-grid h2 {
    border-bottom: 1px solid Black;
    border-top: 1px solid Black;
    padding: 5px;
    margin: 0px;
}

.panel-grid h2 {
    border-right: 1px solid Black;
}

.panel-body {
    overflow: auto;
    padding: 5px;
}

/****************************************
 * footer
 ****************************************/

#footer {
    border-top: 1px solid lightgray;
    bottom: 0;
    font-size: 9pt;
    height: 20px;
    left: 0;
    line-height: 20px;
    margin: 0;
    position: absolute;
    width: 100%;
}

/******************************
 * feedback
 ******************************/

#feedback-dialog {
    display: none;
}

#feedback-dialog p {
    margin-top: 1em;
}

#feedback-dialog .red {
    color: red;
    font-weight: bold;
}

#feedback-dialog .field-wrapper {
    margin-top: 1em;
    padding: 0;
}

#feedback-dialog .field {
    margin-bottom: 0;
    margin-top: 0.5em;
}

#feedback-dialog .referrer .field {
    clear: both;
    float: none;
    margin-top: 1em;
}

#feedback-dialog textarea {
    width: auto;
}
