
/****************************************
 * Global styles for mobile templates
 ****************************************/

/* main user menu button when root */
[data-role="header"] a.root-user,
[data-role="header"] a.root-user:hover {
    background-color: red;
}

/* become/stop root menu links */
#usermenu .root-user a {
    background-color: red;
}

/* normal flash messages */
.flash {
    color: green;
    margin-bottom: 1em;
}

/* error flash messages */
.error,
.error-messages {
    color: red;
    margin-bottom: 1em;
}

/* receiving warning flash messages */
.receiving-warning {
    color: red;
}

.replacement-header {
    display: none;
}

.field-wrapper.with-error {
    background-color: #ddcccc;
    border: 2px solid #dd6666;
    margin-bottom: 1em;
}

.field-wrapper label {
    font-weight: bold;
    margin-top: 1em;
}

.field-error .error-msg {
    color: Red;
}

/* make sure space comes between simple filter and "grid" list */
.simple-filter {
    margin-bottom: 1.5em;
}
