
table.diff {
    background-color: White;
    border-collapse: collapse;
    border-left: 1px solid Black;
    border-top: 1px solid Black;
    font-size: 11pt;
    /* margin-top: 2em; */
    /* margin-left: 50px; */
    min-width: 80%;
}

.ui-dialog-content table.diff {
    color: black;
}

table.diff th,
table.diff td {
    border-bottom: 1px solid Black;
    border-right: 1px solid Black;
    padding: 2px 5px;
}

table.diff td {
    padding: 5px 10px;
}

table.diff.monospace td.old-value,
table.diff.monospace td.new-value{
    font-family: monospace;
    white-space: pre;
}

table.diff.new td.new-value,
table.diff.dirty tr.diff td.new-value {
    background-color: #cfc;
}

table.diff.deleted td.old-value,
table.diff.dirty tr.diff td.old-value {
    background-color: #fcc;
}
